/*
 *  Copyright 2019-2025 Zheng Jie
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package com.fuyuanshen.modules.security.service;

import cn.hutool.jwt.JWT;
import cn.hutool.jwt.JWTUtil;
import com.fuyuanshen.exception.BadRequestException;
import com.fuyuanshen.modules.security.service.dto.AuthorityDto;
import com.fuyuanshen.modules.security.service.dto.JwtUserDto;
import com.fuyuanshen.modules.system.domain.User;
import com.fuyuanshen.modules.system.domain.app.APPUser;
import com.fuyuanshen.modules.system.service.DataService;
import com.fuyuanshen.modules.system.service.RoleService;
import com.fuyuanshen.modules.system.service.UserService;
import com.fuyuanshen.utils.SecurityUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author Zheng Jie
 * @date 2018-11-22
 */
@Slf4j
@RequiredArgsConstructor
@Service("userDetailsService")
public class UserDetailsServiceImpl implements UserDetailsService {

    private final UserService userService;
    private final RoleService roleService;
    private final DataService dataService;
    private final UserCacheManager userCacheManager;

    private final static String APP_USER_TYPE = "1"; // app用户类型

    private final static String SYSTEM_USER_TYPE = "0"; // 系统用户类型
    @Override
    public JwtUserDto loadUserByUsername(String username) {
        if(SecurityUtils.getToken() != null){
            JWT jwt = JWTUtil.parseToken(SecurityUtils.getToken());
            String  userType = jwt.getPayload("userType").toString();
            if(APP_USER_TYPE.equals(userType)){
                return loadUserByAppUsername(username);
            }else{
                return loadSystemUserByUsername(username);
            }
        }else{
            return loadSystemUserByUsername(username);
        }
    }

    /**
     * 加载系统用户详情信息
     * @param username
     * @return
     */
    private JwtUserDto loadSystemUserByUsername(String username) {
        JwtUserDto jwtUserDto = userCacheManager.getUserCache(username);
        if (jwtUserDto == null) {
            User user = userService.getLoginData(username);
            if (user == null) {
                throw new BadRequestException("用户不存在");
            } else {
                if (!user.getEnabled()) {
                    throw new BadRequestException("账号未激活！");
                }
                // 获取用户的权限
                List<AuthorityDto> authorities = roleService.buildPermissions(user);
                // 初始化JwtUserDto
                jwtUserDto = new JwtUserDto(null,user, dataService.getDeptIds(user), authorities);
                // 添加缓存数据
                userCacheManager.addUserCache(username, jwtUserDto);
            }
        }
        return jwtUserDto;
    }

    /**
     * 加载app用户详情信息
     * @param username
     * @return
     */
    public JwtUserDto loadUserByAppUsername(String username) {

        JwtUserDto jwtUserDto = userCacheManager.getAppUserCache(username);
        if (jwtUserDto == null) {
            APPUser user = userService.appGetLoginData(username);
            if (user == null) {
                throw new BadRequestException("用户不存在");
            } else {
                if (!user.getEnabled()) {
                    throw new BadRequestException("账号未激活！");
                }
                // 获取用户的权限
                List<AuthorityDto> authorities = roleService.appBuildPermissions(user);
                // 初始化JwtUserDto
                jwtUserDto = new JwtUserDto(user,null, dataService.getDeptIds(user), authorities);
                // 添加缓存数据
                userCacheManager.addAppUserCache(username, jwtUserDto);
            }
        }
        return jwtUserDto;
    }


}
