package com.fuyuanshen.modules.system.listener.excel;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.lang.UUID;
import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.read.listener.ReadListener;
import com.alibaba.excel.util.ListUtils;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import com.fuyuanshen.modules.system.domain.Device;
import com.fuyuanshen.modules.system.domain.dto.DeviceExcelImportDTO;
import com.fuyuanshen.modules.system.service.DeviceService;
import org.apache.poi.openxml4j.opc.OPCPackage;
import org.apache.poi.openxml4j.util.ZipSecureFile;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.xssf.usermodel.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;

/**
 * @author: 默苍璃
 * @date: 2025-06-0710:15
 */
@Slf4j
public class UploadDeviceDataListener1 implements ReadListener<DeviceExcelImportDTO> {

    /**
     * 每隔5条存储数据库，实际使用中可以100条，然后清理list ，方便内存回收
     */
    private static final int BATCH_COUNT = 100;

    /**
     * 缓存的数据
     */
    private List<Device> cachedDataList = ListUtils.newArrayListWithExpectedSize(BATCH_COUNT);

    /**
     * 假设这个是一个DAO，当然有业务逻辑这个也可以是一个service。当然如果不用存储这个对象没用。
     */
    private DeviceService deviceService;
    private MultipartFile file;


    // 新增一个字段用于存储解析出的设备数据及其索引
    private List<DeviceExcelImportDTO> importDTOList = new ArrayList<>();
    private List<Device> deviceListForImageMapping = new ArrayList<>();


    /**
     * 如果使用了spring,请使用这个构造方法。每次创建Listener的时候需要把spring管理的类传进来
     *
     * @param deviceService
     */
    public UploadDeviceDataListener1(DeviceService deviceService, MultipartFile file) {
        this.deviceService = deviceService;
        this.file = file;
    }

    /**
     * 这个每一条数据解析都会来调用
     *
     * @param data    one row value. It is same as {@link AnalysisContext#readRowHolder()}
     * @param context analysis context
     */
    @Override
    public void invoke(DeviceExcelImportDTO data, AnalysisContext context) {
        log.info("解析到一条数据: {}", JSON.toJSONString(data));

        Device device = new Device();
        BeanUtil.copyProperties(data, device, true);
        cachedDataList.add(device);
        importDTOList.add(data); // 存储原始导入数据
        deviceListForImageMapping.add(device);

        if (cachedDataList.size() >= BATCH_COUNT) {
            saveData();
            cachedDataList = ListUtils.newArrayListWithExpectedSize(BATCH_COUNT);
            deviceListForImageMapping.clear();
        }
    }


    /**
     * 所有数据解析完成了 都会来调用
     *
     * @param context
     */
    @Override
    public void doAfterAllAnalysed(AnalysisContext context) {
        // 这里也要保存数据，确保最后遗留的数据也存储到数据库
        saveData();
        imageProcess(file);
        log.info("所有数据解析完成！");
    }


    /**
     * 加上存储数据库
     */
    private void saveData() {
        log.info("{}条数据，开始存储数据库！", cachedDataList.size());
        deviceService.saveBatch(cachedDataList);
        log.info("存储数据库成功！");
    }


    /**
     * 处理图片
     *
     * @param file
     */
    public void imageProcess(MultipartFile file) {
        if (file == null) {
            log.warn("文件为空，无法处理图片");
            return;
        }

        try {
            ZipSecureFile.setMinInflateRatio(-1.0d);
            OPCPackage opcPackage = OPCPackage.open(file.getInputStream());
            XSSFWorkbook workbook = new XSSFWorkbook(opcPackage);

            for (Sheet sheet : workbook) {
                XSSFSheet xssfSheet = (XSSFSheet) sheet;
                XSSFDrawing drawing = xssfSheet.getDrawingPatriarch();
                if (drawing == null) continue;

                for (XSSFShape shape : drawing.getShapes()) {
                    if (shape instanceof XSSFPicture) {
                        XSSFPicture picture = (XSSFPicture) shape;
                        XSSFClientAnchor anchor = (XSSFClientAnchor) picture.getAnchor();
                        int rowIndex = anchor.getRow1(); // 获取图片所在行
                        int colIndex = anchor.getCol1();  // 获取图片所在列

                        // 假设图片在第 5 列（索引从0开始）
                        if (colIndex == 4 && rowIndex >= 1 && rowIndex < deviceListForImageMapping.size() + 1) {
                            Device device = deviceListForImageMapping.get(rowIndex - 1);
                            byte[] imageData = picture.getPictureData().getData();

                            // TODO: 上传图片并生成URL，这里假设你有一个工具方法 uploadAndGenerateUrl()
                            String imageUrl = uploadAndGenerateUrl(imageData);
                            device.setDevicePic(imageUrl); // 设置图片URL到设备对象
                        }
                    }
                }
            }

            // 更新数据库中的设备图片字段
            deviceService.updateBatchById(deviceListForImageMapping);

        } catch (Exception e) {
            log.error("处理图片失败：{}", e.getMessage());
        }
    }

    // 模拟上传图片并返回URL（实际应替换为你的OSS/FTP等上传逻辑）
    private String uploadAndGenerateUrl(byte[] imageData) {
        // 示例：上传到OSS并返回URL
        // 这里可以调用你自己的文件上传服务
        return "https://example.com/images/" + UUID.randomUUID() + ".jpg";
    }


}
