/*
 *  Copyright 2019-2025 Zheng Jie
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package com.fuyuanshen.modules.system.rest.app;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fuyuanshen.annotation.Log;
import com.fuyuanshen.annotation.rest.AnonymousPostMapping;
import com.fuyuanshen.exception.BadRequestException;
import com.fuyuanshen.modules.system.domain.app.APPUser;
import com.fuyuanshen.modules.system.domain.dto.DeviceForm;
import com.fuyuanshen.modules.system.domain.dto.UserQueryCriteria;
import com.fuyuanshen.modules.system.domain.dto.app.APPForgotPasswordDTO;
import com.fuyuanshen.modules.system.domain.dto.app.APPUpdateUserDTO;
import com.fuyuanshen.modules.system.domain.dto.app.APPUserDTO;
import com.fuyuanshen.modules.system.domain.vo.APPUserVo;
import com.fuyuanshen.modules.system.enums.UserType;
import com.fuyuanshen.modules.system.service.app.APPUserService;
import com.fuyuanshen.modules.utils.ResponseVO;
import com.fuyuanshen.utils.PageResult;
import com.fuyuanshen.utils.SecurityUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.annotations.Param;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * @author Zheng Jie
 * @date 2018-11-23
 */
@Slf4j
@Api(tags = "APP：用户管理")
@RestController
@RequestMapping("/api/appUser")
@RequiredArgsConstructor
public class APPUserController {

    private final APPUserService appUserService;


    @ApiOperation("查询APP用户")
    @GetMapping
    @PreAuthorize("@el.check('appUser:list')")
    public ResponseVO<PageResult<APPUser>> queryAPPUser(UserQueryCriteria criteria) {
        criteria.setUserType(UserType.APP.getValue());
        Page<APPUser> page = new Page<>(criteria.getPage(), criteria.getSize());
        return ResponseVO.success(appUserService.queryAPPUser(criteria, page));
    }


    @ApiOperation("用户中心")
    @GetMapping(value = "/get")
    @PreAuthorize("@el.check('appUser:get')")
    public ResponseVO<APPUserVo> getAPPUser() {
        String userName = SecurityUtils.getCurrentUsername();
        APPUser appUser = appUserService.getLoginData(userName);

        APPUserVo appUserVo = new APPUserVo();
        appUserVo.setId(appUser.getId());
        appUserVo.setNickName(appUser.getNickName());
        appUserVo.setGender(appUser.getGender());
        appUserVo.setPhone(appUser.getPhone());

        return ResponseVO.success(appUserVo);
    }

    @Log("app用户注册")
    @ApiOperation("app用户注册")
    @AnonymousPostMapping(value = "/register")
    public ResponseVO<String> APPRegister(@Validated @RequestBody APPUserDTO user) throws Exception {

        //暫定0000
        if (!"0000".equals(user.getVerificationCode())) {
            throw new BadRequestException("验证码错误");
        }
        appUserService.addUser(user);
        return ResponseVO.success("success！！！");
    }

    @Log("发送用户注册验证码")
    @ApiOperation("发送用户注册验证码")
    @AnonymousPostMapping(value = "/sendRegisterSms")
    public ResponseVO<String> sendRegisterSms(@Param("phoneNumber") String phoneNumber) throws Exception {
//        appUserService.sendSms(phoneNumber);
        return ResponseVO.success("success！！！");
    }

    @Log("修改APP用户")
    @ApiOperation("修改APP用户")
    @PutMapping
    @PreAuthorize("@el.check('appUser:edit')")
    public ResponseVO<String> updateUser(@Validated @ModelAttribute APPUpdateUserDTO appUser) throws Exception {
        Long userId = SecurityUtils.getCurrentUserId();
        if(!userId.equals(appUser.getId())){
            throw new BadRequestException("不能修改他人资料");
        }

        appUserService.updateUser(appUser);
        return ResponseVO.success("success！！！");
    }


    @Log("忘记密码")
    @ApiOperation("忘记密码")
    @AnonymousPostMapping(value = "/forgotPassword")
    public ResponseVO<String> forgotPassword(@RequestBody APPForgotPasswordDTO appForgotPasswordDTO) throws Exception {
        if (!"0000".equals(appForgotPasswordDTO.getVerificationCode())) {
            throw new BadRequestException("验证码错误");
        }
        appUserService.forgotPassword(appForgotPasswordDTO);
        return ResponseVO.success("success！！！");
    }

    @Log("发送忘记密码验证码")
    @ApiOperation("发送忘记密码验证码")
    @AnonymousPostMapping(value = "/sendForgotPasswordSms")
    public ResponseVO<String> sendForgotPasswordSms(@Param("phoneNumber") String phoneNumber) throws Exception {
//        appUserService.sendSms(phoneNumber);
        return ResponseVO.success("success！！！");
    }
}
