package com.fuyuanshen.modules.system.service.app;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fuyuanshen.exception.BadRequestException;
import com.fuyuanshen.modules.system.domain.Device;
import com.fuyuanshen.modules.system.domain.DeviceType;
import com.fuyuanshen.modules.system.domain.app.APPDevice;
import com.fuyuanshen.modules.system.domain.app.APPDeviceType;
import com.fuyuanshen.modules.system.domain.dto.DeviceQueryCriteria;
import com.fuyuanshen.modules.system.domain.dto.app.APPUnbindDTO;
import com.fuyuanshen.modules.system.enums.BindingStatusEnum;
import com.fuyuanshen.modules.system.enums.CommunicationModeEnum;
import com.fuyuanshen.modules.system.enums.UserType;
import com.fuyuanshen.modules.system.mapper.DeviceMapper;
import com.fuyuanshen.modules.system.mapper.DeviceTypeMapper;
import com.fuyuanshen.modules.system.mapper.app.APPDeviceMapper;
import com.fuyuanshen.modules.system.mapper.app.AppDeviceTypeMapper;
import com.fuyuanshen.utils.PageResult;
import com.fuyuanshen.utils.SecurityUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author: WY
 * @Date: 2025/5/16
 **/
@Slf4j
@Service
@RequiredArgsConstructor
public class APPDeviceServiceImpl extends ServiceImpl<APPDeviceMapper, APPDevice> implements APPDeviceService {

    private final APPDeviceMapper appDeviceMapper;
    private final DeviceMapper deviceMapper;
    private final DeviceTypeMapper deviceTypeMapper;
    private final AppDeviceTypeMapper appDeviceTypeMapper;


    /**
     * APP用户设备列表
     *
     * @param criteria
     */
    @Override
    public PageResult<APPDevice> appDeviceList(Page<APPDevice> page, DeviceQueryCriteria criteria) {
        IPage<APPDevice> devices = appDeviceMapper.appDeviceList(page, criteria);
        return new PageResult<>(devices.getRecords(), devices.getTotal());
    }

    /**
     * APP用户设备类型列表
     *
     * @param criteria
     * @return
     */
    @Override
    public List<APPDeviceType> appTypeList(DeviceQueryCriteria criteria) {
        criteria.setCustomerId(SecurityUtils.getCurrentUserId());
        return appDeviceTypeMapper.appTypeList(criteria);
    }


    /**
     * APP/小程序用户设备绑定
     *
     * @param criteria
     */
    @Override
    @Transactional
    public void appBindDevice(DeviceQueryCriteria criteria) {

        List<Device> devices = new ArrayList<>();

        if (criteria.getCommunicationMode().equals(CommunicationModeEnum.BLUETOOTH.getValue())) {
            devices = deviceMapper.selectList(new QueryWrapper<Device>().eq("device_mac", criteria.getDeviceMac()));
            if (CollectionUtil.isEmpty(devices)) {
                throw new BadRequestException("请先将设备入库！！！");
            }
            List<APPDevice> appDevices = appDeviceMapper.selectList(new QueryWrapper<APPDevice>()
                    .eq("device_mac", criteria.getDeviceMac()).eq("binding_type", UserType.APP.getValue()));
            if (CollectionUtil.isNotEmpty(appDevices)) {
                throw new BadRequestException("该设备已绑定！！！");
            }
        }

        if (criteria.getCommunicationMode().equals(CommunicationModeEnum.FOUR_G.getValue())) {
            devices = deviceMapper.selectList(new QueryWrapper<Device>().eq("device_imei", criteria.getDeviceImei()));
            if (CollectionUtil.isEmpty(devices)) {
                throw new BadRequestException("请先将设备入库！！！");
            }
            List<APPDevice> appDevices = appDeviceMapper.selectList(new QueryWrapper<APPDevice>()
                    .eq("device_imei", criteria.getDeviceImei()).eq("binding_type", UserType.APP.getValue()));
            if (CollectionUtil.isNotEmpty(appDevices)) {
                throw new BadRequestException("该设备已绑定！！！");
            }
        }

        Device device = devices.get(0);
        device.setBindingStatus(BindingStatusEnum.BOUND.getCode());
        deviceMapper.updateById( device);

        APPDevice appDevice = new APPDevice();
        BeanUtil.copyProperties(device, appDevice);
        appDevice.setBindingType(UserType.APP.getValue());
        appDevice.setBindingStatus(BindingStatusEnum.BOUND.getCode());
        Long currentUserId = SecurityUtils.getCurrentUserId();
        appDevice.setCustomerId(currentUserId);
        appDevice.setCreateTime(new Timestamp(System.currentTimeMillis()));
        // 设备类型名称
        appDevice.setDeviceTypeName(device.getTypeName());
        appDeviceMapper.insert(appDevice);

        APPDeviceType appDeviceType = appDeviceTypeMapper.selectById(device.getDeviceType());
        if (appDeviceType == null) {
            DeviceType deviceType = deviceTypeMapper.selectById(device.getDeviceType());
            APPDeviceType type = new APPDeviceType();
            BeanUtil.copyProperties(deviceType, type);
            type.setCustomerId(currentUserId);
            appDeviceTypeMapper.insert(type);
        }

    }


    /**
     * 分页查询APP/小程序设备
     *
     * @param criteria
     * @param page
     * @return
     */
    @Override
    public PageResult<APPDevice> queryAll(Page<APPDevice> page, DeviceQueryCriteria criteria) {
        IPage<APPDevice> devices = appDeviceMapper.queryAll(page, criteria);
        return new PageResult<>(devices.getRecords(), devices.getTotal());
    }


    /**
     * APP/小程序用户设备解绑
     *
     * @param deviceForm
     */
    @Override
    @Transactional
    public void unbindAPPDevice(APPUnbindDTO deviceForm) {

        appDeviceMapper.delete(new QueryWrapper<APPDevice>().eq("device_mac", deviceForm.getDeviceMac()).eq("binding_type", UserType.APP.getValue()));
        List<Device> devices = deviceMapper.selectList(new QueryWrapper<Device>().eq("device_mac", deviceForm.getDeviceMac()));
        List<Long> ids = devices.stream()
                .map(Device::getId)
                .collect(Collectors.toList());
        appDeviceTypeMapper.deleteBatchIds(ids);
        Device device = new Device();
        device.setId(deviceForm.getCustomerId());
        device.setBindingStatus(BindingStatusEnum.UNBOUND.getCode());
        deviceMapper.updateById(device);
    }


}
